<?php
/**
 *       @file  qtrScanLock.php
 *      @brief  This module contains implelemtation of a scan lock to prevent concurrent execution of multiple scanners
 *
 *     @author  Quttera (qtr), contactus@quttera.com
 *
 *   @internal
 *     Created  01/20/2016
 *    Compiler  gcc/g++
 *     Company  Quttera
 *   Copyright  Copyright (c) 2016, Quttera
 *
 * This source code is released for free distribution under the terms of the
 * GNU General Public License as published by the Free Software Foundation.
 * =====================================================================================
 */

class CQtrScanLock
{

    public static function Acquire()
    {
        if( self::IsLocked() == TRUE ){
            /* 
             * Lock already taken
             */
            return FALSE;
        }

        return self::Lock();
    }


    public static function TryAcquire()
    {
        return self::Acquire();
    }


    public static function IsLocked()
    {
        return is_file( self::LockName() );
    }

    public static function Lock()
    {
        $lock = @fopen(self::LockName(),"w");
        if( !$lock ){
            /*
             * Failed to open lock
             */
            return FALSE;
        }
        @fwrite($lock,sprintf("%d",getmypid()));
        @fflush($lock);
        @fclose($lock);
        return self::IsLocked();
    }

    public static function Release()
    {
        @unlink( self::LockName() );
        return TRUE;
    }

    public static function LockName()
    {
        $tmp_path       = NULL;

        if( function_exists(get_temp_dir) ){
            /*
             * WordPress internal implementation 
             */
            $tmp_path = get_temp_dir();
        }else{
            $tmp_path = @sys_get_temp_dir();
        }

        $lock_file_name = "__QTR_LOCK__";
        return $tmp_path . DIRECTORY_SEPARATOR . $lock_file_name;
    }

    public static function ForceUnlock(){
        return self::Release();
    }

}

?>
